/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/

#include <tpf/i_ecb3.h>
#include "trag.h"

void test() {

    /* Get the parameters from the ebw area */
    int restarts_left = *( (int *) &ecbptr()->ebw000 );
    int restarts_before_switch = *( (int *) &ecbptr()->ebw004 );

    /* If restarts_before_switch is 0, it's time to switch over to the BSO */
    if ( restarts_before_switch == 0 ) {
	
	/* Alter the BSO's tracegroup to make sure it is not IBM_DEFT */
	alter_tracegroup( BSO_STRING );
	
	/* Decrement the number of restarts left */
	( *( (int *) &ecbptr()->ebw000 ) )--;	
	
	/* Set restarts_before_switch to -1 since this is the only switch */
	( *( (int *) &ecbptr()->ebw004 ) )--;
	
	/* Jump to the BSO */
	entdc( BSO_PROGRAM, NULL );
    }
    /*******************************************************************/
    /* If the number of restarts is less than the number of buffers,   */
    /* make another tracegroup.                                        */
    /*******************************************************************/
    else if ( restarts_left > 0 ) {
	
        /* Change the tracegroup for this program */
	alter_tracegroup( CSO_STRING );
	
	/* Decrement the number of restarts left */
	( *( (int *) &ecbptr()->ebw000 ) )--;
	
	/*******************************************************************/
	/* If restarts_before_switch is not -1, we'll be switching to the  */
	/* BSO later, so decrement the counter                             */
	/*******************************************************************/
	if ( restarts_before_switch != NO_SWITCH ) {
	    ( *( (int *) &ecbptr()->ebw004 ) )--;
	}

	/* Re-enter this program */
	entdc( CSO_PROGRAM, NULL );
    }
    /* Otherwise, this tracegroup should be using the IBM_DEFT buffer */
    else {

	/*******************************************************************/
	/* Verify that the correct buffers are being used.                 */
	/* Both the current C function trace buffer and the current macro  */
	/* trace buffer should be the first buffers in the list since      */
	/* the ECB is now using the IBM_DEFT buffers.                      */
	/*******************************************************************/
	struct ictrace_header *current_c_buffer =
	    (struct ictrace_header *) tpf_ecbp3()->ce3curch;
	struct ictrace_header *current_macro_buffer =
	    (struct ictrace_header *) tpf_ecbp3()->ce3curmh;


	struct iecb_tr_buffer_list *buffer_list =
	    (struct iecb_tr_buffer_list *) ecbp2()->ce2trlist;

	struct ictrace_header *first_c_buffer = buffer_list->cfunc_tr_buf;
	struct ictrace_header *first_macro_buffer = buffer_list->macro_tr_buf;


	/* Make sure the buffers are the same- otherwise there is an error */
	if ( current_c_buffer == first_c_buffer &&
	     current_macro_buffer == first_macro_buffer ) {

	    wtopc( "CORRECT BUFFERS USED", WTOPC_PRC,
		   WTOPC_NO_CHAIN, NULL );

	} else {

	    struct wtopc_header header_buffer;
	    wtopc_insert_header( &header_buffer, NULL, 1, WTOPC_ERROR,
			     WTOPC_SYS_TIME );

	    wtopc( "ERROR: INCORRECT BUFFERS USED", WTOPC_PRC,
		   WTOPC_NO_CHAIN, &header_buffer );

	}
    }

    exit( EXIT_SUCCESS );
}


void alter_tracegroup( const char *const program_name ) {

    /* Change the protection key to allow writing to the PAT */
    cinfc( CINFC_WRITE, CINFC_CMMPAT );

    /* Obtain a pointer to the PAT */
    struct pat *pat_ptr = progc( program_name, PROGC_PBI );

    /* Verify that the PAT was successfully located */
    if ( pat_ptr == NULL ) {
	
	/* If not, send an error message and exit */
	char text_buffer[BUFFER_SIZE];
	sprintf( text_buffer, "ERROR LOCATING PAT FOR PROGRAM %s",
		 program_name );
	
	struct wtopc_header header_buffer;
	wtopc_insert_header( &header_buffer, NULL, 0, WTOPC_ERROR,
			     WTOPC_SYS_TIME );
	
	wtopc( text_buffer, WTOPC_PRC, WTOPC_NO_CHAIN, &header_buffer );
	
	exit( EXIT_FAILURE );
    }

    /* Alter the tracegroup index and name */
    unsigned short index = ++( pat_ptr->pattrindx );	
    sprintf((char *) pat_ptr->pattrgrp, "TG%d", index );

    /* Restore the original protection key */
    keyrc();
}
