/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/

#include <tpf/sysapi.h>
#include <sys/wait.h>
#include "trag.h"

/***************************************************************************/
/* Driver to confirm that an ECB will use the first tracegroup buffer      */
/* (the IBM_DEFT buffer) after exceeding the maximum number of tracegroup  */
/* buffers.  All 4 variations between CSO and BSO are tested for the       */
/* transition to the final tracegroup (CSO to CSO, CSO to BSO, BSO to BSO, */
/* and BSO to CSO).  A new ECB is created for each case, with 2 parameters */
/* passed to the ECB through the ebw area.  The first parameter is the     */
/* total number of times the ECB must change its tracegroup and restart    */
/* (equal to the number of buffers), and the second parameter is the       */
/* number of restarts left before the ECB should jump to the other type    */
/* of program (CSO or BSO).  For the second parameter, -1 represents no    */
/* switch.                                                                 */
/***************************************************************************/
void test() {

    struct tpf_fork_input create_parameters;
    pid_t child_pid;


    /* CSO to CSO Case */

    wtopc( "TESTING CSO TO CSO", WTOPC_PRC,
	   WTOPC_NO_CHAIN, NULL );

    /* Give the wtopc a chance to complete */
    tpf_yieldc( YIELDC_VCT_LIST );

    /* Set up the input struct for tpf_fork */
    create_parameters.program = CSO_STRING;
    create_parameters.prog_type = TPF_FORK_NAME;
    create_parameters.istream = TPF_FORK_IS_BALANCE;

    /*******************************************************************/
    /* Set up the appropriate data to be passed to the child ECB.      */
    /* Get ECB Page 2 to determine the number of tracegroup buffers    */
    /* to use as the total number of "restarts" for the child ECB.     */
    /* Use -1 for the second parameter since there is no jump to the   */
    /* assembly program for this case.                                 */
    /*******************************************************************/
    int data[] = { ecbp2()->ce2trmax, NO_SWITCH };

    create_parameters.ebw_data_length = PARAMETER_LENGTH;
    create_parameters.ebw_data = data;
    create_parameters.parm_data = NULL;

    /* Create the new ECB and wait for it to finish executing */
    child_pid = tpf_fork( &create_parameters );
    waitpid( child_pid, NULL, 0 );


    /* CSO to BSO Case */

    wtopc( "TESTING CSO TO BSO", WTOPC_PRC,
	   WTOPC_NO_CHAIN, NULL );

    /* Give the wtopc a chance to complete */
    tpf_yieldc( YIELDC_VCT_LIST );

    /***********************************************************************/
    /* Change the second parameter to (tracegroup buffers - 1) so that the */
    /* CSO will switch to the BSO for the last tracegroup.                 */
    /***********************************************************************/
    data[1] = data[0] - 1;

    /* Create the new ECB and wait for it to finish executing */
    child_pid = tpf_fork( &create_parameters );
    waitpid( child_pid, NULL, 0 );


    /* BSO to BSO Case */

    wtopc( "TESTING BSO TO BSO", WTOPC_PRC,
	   WTOPC_NO_CHAIN, NULL );

    /* Give the wtopc a chance to complete */
    tpf_yieldc( YIELDC_VCT_LIST );

    /* Execute the BSO and change the second parameter so there is no switch */
    create_parameters.program = BSO_STRING;
    data[1] = NO_SWITCH;

    /* Create the new ECB and wait for it to finish executing */
    child_pid = tpf_fork( &create_parameters );
    waitpid( child_pid, NULL, 0 );


    /* BSO to CSO Case */

    wtopc( "TESTING BSO TO CSO", WTOPC_PRC,
	   WTOPC_NO_CHAIN, NULL );

    /* Give the wtopc a chance to complete */
    tpf_yieldc( YIELDC_VCT_LIST );

    /* Alter the second parameter to cause a switch to the CSO */
    data[1] = data[0] - 1;

    /* Create the new ECB and wait for it to finish executing */
    child_pid = tpf_fork( &create_parameters );
    waitpid( child_pid, NULL, 0 );


    exit( EXIT_SUCCESS );
}
